@@ -1,6 +1,6 @@
 # Model menu dialogs
 
-# last modified 2015-09-04 by J. Fox
+# last modified 2017-02-07 by J. Fox
 
 selectActiveModel <- function(){
 	models <- listAllModels()
@@ -232,12 +232,14 @@ anovaTable <- function () {
     .activeModel <- ActiveModel()
     if (is.null(.activeModel)) 
         return()
-    defaults <- list (initial.type = "II", initial.sandwich="0", initial.sandwich.type="HC3")
+    aov <- modelCapability("aov")
+    Aov <- modelCapability("Aov")
+    defaults <- list (initial.type = if (Aov) "II" else "I", initial.sandwich="0", initial.sandwich.type="HC3")
     dialog.values <- getDialog ("anovaTable", defaults)
     initializeDialog(title = gettextRcmdr("ANOVA Table"))
-    radioButtons(name = "type", buttons = c("I", "II", "III"), 
-        values = c("I", "II", "III"), labels = gettextRcmdr(c("Sequential (\"Type I\")", 
-            "Partial obeying marginality (\"Type II\")", "Partial ignoring marginality (\"Type III\")")), 
+    radioButtons(name = "type", buttons = c(if (aov) "I", if(Aov) c("II", "III")), 
+        values = c(if (aov) "I", if (Aov) c("II", "III")), labels = gettextRcmdr(c(if (aov) "Sequential (\"Type I\")", 
+            if (Aov) c("Partial obeying marginality (\"Type II\")", "Partial ignoring marginality (\"Type III\")"))), 
         title = gettextRcmdr("Type of Tests"), initialValue = dialog.values$initial.type)
     sandwichVar <- tclVar(dialog.values$initial.sandwich)
     sandwichFrame <- tkframe(top)
@@ -323,6 +325,7 @@ VIF <- function(){
 	.activeModel <- ActiveModel()
 	if (is.null(.activeModel) || !checkMethod("vif", .activeModel, default=TRUE)) return()
 	doItAndPrint(paste("vif(", .activeModel, ")", sep=""))
+	doItAndPrint(paste("round(cov2cor(vcov(", .activeModel, ")), 3) # Correlations of parameter estimates", sep=""))
 }
 
 addObservationStatistics <- function () {
@@ -351,32 +354,38 @@ addObservationStatistics <- function () {
   initializeDialog(title = gettextRcmdr("Add Observation Statistics to Data"))
   .variables <- Variables()
   obsNumberExists <- is.element("obsNumber", .variables)
-  activate <- c(checkMethod("fitted", .activeModel, default = TRUE, 
-                            reportError = FALSE), checkMethod("residuals", .activeModel, 
-                                                              default = TRUE, reportError = FALSE), checkMethod("rstudent", 
-                                                                                                                .activeModel, reportError = FALSE), checkMethod("hatvalues", 
-                                                                                                                                                                .activeModel, reportError = FALSE), checkMethod("cooks.distance", 
-                                                                                                                                                                                                                .activeModel, reportError = FALSE))
+  # activate <- c(checkMethod("fitted", .activeModel, default = TRUE, 
+  #                           reportError = FALSE), checkMethod("residuals", .activeModel, 
+  #                                                             default = TRUE, reportError = FALSE), checkMethod("rstudent", 
+  #                                                                                                               .activeModel, reportError = FALSE), checkMethod("hatvalues", 
+  #                                                                                                                                                               .activeModel, reportError = FALSE), checkMethod("cooks.distance", 
+  #                                                                                                                                                                                                              .activeModel, reportError = FALSE))
+  activate <- c(modelCapability("fit"), modelCapability("res"), modelCapability("rst"), modelCapability("hat"), modelCapability("cook"))
   checkBoxes(frame = "selectFrame", boxes = c(c("fitted", "residuals", 
                                                 "rstudent", "hatvalues", "cookd")[activate], "obsNumbers"), 
              labels = c(gettextRcmdr(c("Fitted values", "Residuals", 
                                        "Studentized residuals", "Hat-values", "Cook's distances"))[activate], 
-                        gettextRcmdr("Observation indices")), initialValues = c(dialog.values$initial.fitted, 
+                        gettextRcmdr("Observation indices")), initialValues = c(c(dialog.values$initial.fitted, 
                                                                                 dialog.values$initial.residuals, dialog.values$initial.rstudent, 
-                                                                                dialog.values$initial.hatvalues, dialog.values$initial.cookd, dialog.values$initial.obsNumbers))
+                                                                                dialog.values$initial.hatvalues, dialog.values$initial.cookd)[activate], dialog.values$initial.obsNumbers))
   command <- paste(.activeDataSet, "<- within(", .activeDataSet, ", {", sep="")
   onOK <- function() {
     closeDialog()
     if (activate[1] && tclvalue(fittedVariable) == 1) 
       command <- paste(command, "\n  ", addVariable("fitted"), sep="")
+    else fittedVariable <- tclVar(0)
     if (activate[2] && tclvalue(residualsVariable) == 1) 
       command <- paste(command, "\n  ", addVariable("residuals"), sep="")
+    else residualsVariable <- tclVar(0)
     if (activate[3] && tclvalue(rstudentVariable) == 1) 
       command <- paste(command, "\n  ", addVariable("rstudent"), sep="")
+    else rstudentVariable <- tclVar(0)
     if (activate[4] && tclvalue(hatvaluesVariable) == 1) 
       command <- paste(command, "\n  ", addVariable("hatvalues"), sep="")
+    else hatvaluesVariable <- tclVar(0)
     if (activate[5] && tclvalue(cookdVariable) == 1) 
       command <- paste(command, "\n  ", addVariable("cooks.distance"), sep="")
+    else cookdVariable <- tclVar(0)
     obsNumbers <- tclvalue(obsNumbersVariable)
     putDialog ("addObservationStatistics", list (initial.fitted = tclvalue (fittedVariable),
                                                  initial.residuals = tclvalue (residualsVariable), initial.rstudent = tclvalue(rstudentVariable), 
@@ -973,7 +982,7 @@ subsetRegression <- function () {
 }
 
 effectPlots <- function () {
-  Library("effects")
+#  Library("effects")
   defaults <- list(initial.all.or.pick = "TRUE", initial.predictors = NULL, 
                    initial.partial.res = 0, initial.span = 50, initial.style = "stacked")
   dialog.values <- getDialog("effectPlots", defaults)
@@ -1064,14 +1073,192 @@ effectPlots <- function () {
   dialogSuffix()
 }
 
-# effectPlots <- function(){
-#   Library("effects")
-#   .activeModel <- ActiveModel()
-#   if (is.null(.activeModel) || !checkMethod("Effect", .activeModel)) return()
-#   doItAndPrint('trellis.device(theme="col.whitebg")')
-#   command <- paste("plot(allEffects(", .activeModel, "), ask=FALSE)", sep="")
-#   justDoIt(command)
-#   logger(command)
-#   activateMenus()
-#   NULL
-# }
+Bootstrap <- function () {
+    .activeModel <- ActiveModel()
+    if (is.null(.activeModel)) 
+        return()
+    defaults <- list (initial.level = "0.95", initial.samples="999", initial.type="bca", initial.method="case", initial.plot="1")
+    dialog.values <- getDialog ("Bootstrap", defaults)
+    initializeDialog(title = gettextRcmdr("Bootstrap"))
+    tkgrid(labelRcmdr(top, text = gettextRcmdr("Bootstrap Confidence Intervals for Coefficients"), 
+                      fg = getRcmdr("title.color"), font="RcmdrTitleFont"), sticky = "w")
+    onOK <- function() {
+        opts <- options(warn = -1)
+        on.exit(options(opts))
+        level <- tclvalue(confidenceLevel)
+        level <- as.numeric(level)
+        closeDialog()
+        if ((is.na(level)) || !is.numeric(level) || (level < 0) || (level > 1)) {
+            Message(gettextRcmdr("Confidence level must be a number between 0 and 1."),
+                    type="error")
+            Bootstrap()
+            return()
+        }
+        samples <- tclvalue(bootstrapSamples)
+        samples <- as.numeric(samples)
+        if ((is.na(samples)) || !is.numeric(samples) || (samples < 0) || (samples != floor(samples))) {
+            Message(gettextRcmdr("Number of samples must be a positive integer."),
+                    type="error")
+            Bootstrap()
+            return()
+        }
+        method <- if (lm){
+            tclvalue(methodVariable)
+        }
+        else defaults$initial.method
+        type <- tclvalue(typeVariable)
+        plot.samples <- tclvalue(plotVar)
+        putDialog ("Bootstrap", list (initial.level = level, initial.samples = samples, initial.type=type, initial.method = method, 
+                                      initial.plot=plot.samples))
+        command <- if (lm) 
+            paste0('Boot(', .activeModel, ', R=', samples, ', method="', method, '")')
+        else paste0("Boot(", .activeModel, ', R=', samples, ')')
+        
+        if (plot.samples == "0"){
+            command <- paste0("confint(", command, ',  level=', level, ', type="', type, '")')
+            doItAndPrint(command)
+            return()
+        }
+        
+        command <- paste0(".bs.samples <- ", command, "\n",
+                          "plotBoot(.bs.samples)")
+        doItAndPrint(command)
+        doItAndPrint(paste0('confint(.bs.samples, level=', level, ', type="', type, '")'))
+        logger("remove(.bs.samples)")
+        justDoIt("remove(.bs.samples, envir=.GlobalEnv)")
+        tkfocus(CommanderWindow())
+    }
+    OKCancelHelp(helpSubject = "Boot", reset = "Bootstrap", apply = "Bootstrap")
+    confidenceFrame <- tkframe(top)
+    confidenceLevel <- tclVar(dialog.values$initial.level)
+    confidenceField <- ttkentry(confidenceFrame, width = "6", textvariable = confidenceLevel)
+    samplesFrame <- tkframe(top)
+    bootstrapSamples <- tclVar(dialog.values$initial.samples)
+    samplesField <- ttkentry(samplesFrame, width = "6", 
+                             textvariable = bootstrapSamples)
+    tkgrid(labelRcmdr(confidenceFrame, text = gettextRcmdr("Confidence level:")), 
+           confidenceField, sticky = "w")
+    tkgrid(confidenceFrame, sticky = "w")
+    tkgrid(labelRcmdr(samplesFrame, text = gettextRcmdr("Number of bootstrap samples:")), 
+           samplesField, sticky = "w")
+    tkgrid(samplesFrame, sticky = "w")
+    radioButtons(name="type", buttons = c("bca", "norm", "basic", "perc"), initialValue=dialog.values$initial.type,
+                 labels = gettextRcmdr(c("BCa (adjusted percentile)", "Normal theory", "Basic bootstrap", "Percentile")),
+                 title=gettextRcmdr("Type of Confidence Intervals"))
+    tkgrid(typeFrame, sticky="w")
+    lm <- lmP()
+    if (lm) {
+        radioButtons(name = "method", buttons = c("case", "residual"), initialValue=dialog.values$initial.method,
+                     labels = gettextRcmdr(c("Case resampling", "Residual resampling")), title = gettextRcmdr("Resampling Method"))
+        tkgrid(methodFrame, sticky="w")
+    }
+    plotVar <- tclVar(dialog.values$initial.plot)
+    plotFrame <- tkframe(top)
+    plotCheckBox <- ttkcheckbutton(plotFrame, variable = plotVar)
+    tkgrid(tklabel(plotFrame, text=gettextRcmdr("Plot bootstrap samples"), fg=getRcmdr("title.color")), plotCheckBox, sticky="w")
+    tkgrid(plotFrame, sticky="w")
+    tkgrid(buttonsFrame, sticky = "w")
+    dialogSuffix()
+}
+
+DeltaMethodConfInt <- function () {
+    .activeModel <- ActiveModel()
+    if (is.null(.activeModel)) 
+        return()
+    defaults <- list(initial.level="0.95", initial.expression="")
+    dialog.values <- getDialog ("DeltaMethodConfInt", defaults)
+    initializeDialog(title = gettextRcmdr("Delta Method"))
+    tkgrid(labelRcmdr(top, text = paste0(gettextRcmdr("Confidence Interval by the Delta Method"), "\n"), 
+                      fg = getRcmdr("title.color"), font="RcmdrTitleFont"), sticky = "w")
+    onOK <- function() {
+        opts <- options(warn = -1)
+        on.exit(options(opts))
+        level <- tclvalue(confidenceLevel)
+        level <- as.numeric(level)
+        expression <- tclvalue(expressionVar)
+        closeDialog()
+        if ((is.na(level)) || !is.numeric(level) || (level < 0) || (level > 1)) {
+            Message(gettextRcmdr("Confidence level must be a number between 0 and 1."),
+                    type="error")
+            DeltaMethodConfInt()
+            return()
+        }
+        putDialog ("DeltaMethodConfInt", list(initial.level=level, initial.expression=expression))
+        command <- paste0('DeltaMethod(', .activeModel, ', "', expression, '", level=', level, ')')
+        doItAndPrint(command)
+        tkfocus(CommanderWindow())
+    }
+    OKCancelHelp(helpSubject = "DeltaMethod", reset = "DeltaMethodConfInt", apply = "DeltaMethodConfInt")
+    coefs <- coef(get(.activeModel, envir=.GlobalEnv))
+    p <- length(coefs)
+    pars <- names(coefs)
+    nms <- if (pars[1] == "(Intercept)") paste0("b", 0:(p - 1)) else paste0("b", 1:p)
+    mat <- cbind(c("Parameter", pars), c("Name", nms))
+    parTable <- tclArray()
+    for (i in 1:(p + 1))
+        for (j in 1:2)
+            parTable[[i - 1, j - 1]] <- strsplit(mat[i, j], " ", fixed = TRUE)[[1]]
+    tableFrame <- tkframe(top)
+    table <- tk2table(tableFrame, variable = parTable, rows = p + 1, cols = 2, height=min(5, p + 1),
+                      titlerows = 1, selectmode = "extended", colwidth = 25, background = "white")
+    tableYscroll <- ttkscrollbar(tableFrame, orient="vertical", command=function(...) tkyview(table, ...))
+    tkconfigure(table, yscrollcommand=function(...) tkset(tableYscroll, ...))
+    tkgrid(table, tableYscroll, sticky="ns")
+    tkgrid(tableFrame, sticky="w")
+    tkgrid(labelRcmdr(top, text=""))
+    confidenceFrame <- tkframe(top)
+    confidenceLevel <- tclVar(dialog.values$initial.level)
+    confidenceField <- ttkentry(confidenceFrame, width = "6", textvariable = confidenceLevel)
+    tkgrid(labelRcmdr(confidenceFrame, text = gettextRcmdr("Confidence level:")), 
+           confidenceField, sticky = "w")
+    tkgrid(confidenceFrame, sticky = "w")
+    expressionFrame <- tkframe(top)
+    expressionVar <- tclVar(dialog.values$initial.expression)
+    expressionBox <- ttkentry(expressionFrame, font=getRcmdr("logFont"), width="55", textvariable=expressionVar)
+    expressionXscroll <- ttkscrollbar(expressionFrame,
+                                      orient="horizontal", command=function(...) tkxview(expressionBox, ...))
+    tkconfigure(expressionBox, xscrollcommand=function(...) tkset(expressionXscroll, ...))
+    tkgrid(labelRcmdr(expressionFrame, text=paste0("\n", gettextRcmdr("Expression to evaluate; use parameter names:"), " ",
+                                                  if (p <= 3) paste(nms, collapse=", ") 
+                                                  else paste0(paste(nms[1:3], collapse=", "), ", ..."))), 
+           sticky="w")
+    tkgrid(expressionBox, sticky="w")
+    tkgrid(expressionXscroll, sticky="ew")
+    tkgrid(expressionFrame, sticky="nw")
+    tkgrid(buttonsFrame, sticky = "w")
+    dialogSuffix()
+    tkfocus(expressionBox)
+}
+
+compareCoefficients <- function () {
+    defaults <- list(initial.models = NULL)
+    dialog.values <- getDialog ("compareCoefficients", defaults)  
+    models <- listAllModels()
+    if (length(models) < 2) {
+        Message(message = gettextRcmdr("There are fewer than two models."), 
+                type = "error")
+        tkfocus(CommanderWindow())
+        return()
+    }
+    initial.models <- if (length(models) == 2) 0:1 else varPosn(dialog.values$initial.models, vars=models)
+    initializeDialog(title = gettextRcmdr("Compare Model Coefficients"))
+    modelsBox <- variableListBox(top, models, title = gettextRcmdr("Select models (pick two or more)"),
+                                 selectmode = "multiple",
+                                 initialSelection = initial.models)
+    onOK <- function() {
+        models <- getSelection(modelsBox)
+        closeDialog()
+        if (length(models) < 2) {
+            errorCondition(recall = compareCoefficients, message = gettextRcmdr("You must select at least two models."))
+            return()
+        }
+        putDialog ("compareCoefficients", list(initial.models=models))
+        command <- paste0("compareCoefs(", paste(models, collapse=", "), ")")
+        doItAndPrint(command)
+        tkfocus(CommanderWindow())
+    }
+    OKCancelHelp(helpSubject = "compareCoefs", reset = "compareCoefficients", apply = "compareCoefficients")
+    tkgrid(getFrame(modelsBox), sticky = "nw")
+    tkgrid(buttonsFrame, columnspan = 2, sticky = "w")
+    dialogSuffix()
+}
